From c554fad9b73baaa4ce13d7e3be54b3552ea48be2 Mon Sep 17 00:00:00 2001
From: Bin Wu <wu.wubin@huawei.com>
Date: Tue, 27 Oct 2020 19:51:59 +0800
Subject: [PATCH] device_model:Add testcases for set_driver_features and
 get_device_features in block.rs

Signed-off-by: Bin Wu <wu.wubin@huawei.com>
---
 device_model/src/virtio/block.rs | 78 ++++++++++++++++++++++++++++++++
 1 file changed, 78 insertions(+)

diff --git a/device_model/src/virtio/block.rs b/device_model/src/virtio/block.rs
index bdcb691..71b60fd 100644
--- a/device_model/src/virtio/block.rs
+++ b/device_model/src/virtio/block.rs
@@ -978,4 +978,82 @@ mod tests {
         let id_bytes = get_serial_num_config(&serial_num);
         assert_eq!(id_bytes.len(), 20);
     }
+
+    #[test]
+    fn test_set_driver_features() {
+        let mut block = Block::new();
+
+        //If the device feature is 0, all driver features are not supported.
+        block.device_features = 0;
+        let driver_feature: u32 = 0xFF;
+        let page = 0_u32;
+        block.set_driver_features(page, driver_feature);
+        assert_eq!(block.driver_features, 0_u64);
+        assert_eq!(block.get_device_features(0_u32), 0_u32);
+
+        let driver_feature: u32 = 0xFF;
+        let page = 1_u32;
+        block.set_driver_features(page, driver_feature);
+        assert_eq!(block.driver_features, 0_u64);
+        assert_eq!(block.get_device_features(1_u32), 0_u32);
+
+        //If both the device feature bit and the front-end driver feature bit are
+        //supported at the same time,  this driver feature bit is supported.
+        block.device_features = 1_u64 << VIRTIO_F_VERSION_1 | 1_u64 << VIRTIO_F_RING_INDIRECT_DESC;
+        let driver_feature: u32 = (1_u64 << VIRTIO_F_RING_INDIRECT_DESC) as u32;
+        let page = 0_u32;
+        block.set_driver_features(page, driver_feature);
+        assert_eq!(
+            block.driver_features,
+            (1_u64 << VIRTIO_F_RING_INDIRECT_DESC)
+        );
+        assert_eq!(
+            block.get_device_features(page),
+            (1_u32 << VIRTIO_F_RING_INDIRECT_DESC)
+        );
+        block.driver_features = 0;
+
+        block.device_features = 1_u64 << VIRTIO_F_VERSION_1;
+        let driver_feature: u32 = (1_u64 << VIRTIO_F_RING_INDIRECT_DESC) as u32;
+        let page = 0_u32;
+        block.set_driver_features(page, driver_feature);
+        assert_eq!(block.driver_features, 0);
+        assert_eq!(block.get_device_features(page), 0_u32);
+        block.driver_features = 0;
+
+        block.device_features = 1_u64 << VIRTIO_F_VERSION_1 | 1_u64 << VIRTIO_F_RING_INDIRECT_DESC;
+        let driver_feature: u32 = (1_u64 << VIRTIO_F_RING_INDIRECT_DESC) as u32;
+        let page = 0_u32;
+        block.set_driver_features(page, driver_feature);
+        assert_eq!(
+            block.driver_features,
+            (1_u64 << VIRTIO_F_RING_INDIRECT_DESC)
+        );
+        assert_eq!(
+            block.get_device_features(page),
+            (1_u32 << VIRTIO_F_RING_INDIRECT_DESC)
+        );
+
+        block.device_features = 1_u64 << VIRTIO_F_VERSION_1 | 1_u64 << VIRTIO_F_RING_INDIRECT_DESC;
+        let driver_feature: u32 = (1_u64 << VIRTIO_F_RING_INDIRECT_DESC) as u32;
+        let page = 0_u32;
+        block.set_driver_features(page, driver_feature);
+        let driver_feature: u32 = ((1_u64 << VIRTIO_F_VERSION_1) >> 32) as u32;
+        let page = 1_u32;
+        block.set_driver_features(page, driver_feature);
+        assert_eq!(
+            block.driver_features,
+            (1_u64 << VIRTIO_F_VERSION_1 | 1_u64 << VIRTIO_F_RING_INDIRECT_DESC)
+        );
+        let page = 0_u32;
+        assert_eq!(
+            block.get_device_features(page),
+            (1_u32 << VIRTIO_F_RING_INDIRECT_DESC)
+        );
+        let page = 1_u32;
+        assert_eq!(
+            block.get_device_features(page),
+            ((1_u64 << VIRTIO_F_VERSION_1) >> 32) as u32
+        );
+    }
 }
-- 
2.27.0

